const $$ = document.querySelectorAll.bind(document)

/**
 * Accordion with keyboard support
 * Use:
 *  <button data-rds-accordion aria-expanded="<true|false>" aria-controls="#elementoADesplegar">Despliégame</button>
 *  <div id="elementoADesplegar">...</div>
 */
const Accordion = {
  KEYCODE: 32, // Space key
  SELECTOR: 'rds-accordion',

  init: function(selector = this.SELECTOR) {
    this.SELECTOR = selector

    $$(`[data-${this.SELECTOR}]`).forEach(item => {
      item.addEventListener('click', e => this.handleClick.call(this, e, item))
      item.addEventListener('keydown', e => this.handleKeydown.call(this, e, item))

      if (item.getAttribute('aria-expanded') === 'false') {
        this.toggleExpand(item)
      }
    })
  },

  handleClick: function(e, item) {
    e.preventDefault()
    this.toggleExpand(item)
  },

  handleKeydown: function(e, item) {
    if (e.keyCode === this.KEYCODE) {
      this.toggleExpand(item)
      e.stopPropagation()
      e.preventDefault()
    }
  },

  toggleExpand: function(item) {
    item.classList.toggle('closed')
    $$(item.getAttribute('aria-controls')).forEach(i => i.classList.toggle('closed'))

    if (item.classList.contains('closed')) {
      item.setAttribute('aria-expanded', 'false')
    } else {
      item.setAttribute('aria-expanded', 'true')
    }
  }
}

export default Accordion
