const $ = document.querySelector.bind(document)
const $$ = document.querySelectorAll.bind(document)

/**
 * Lightbox with keyboard support
 * Use:
 *  <button type="button" data-rds-lightbox-target="#modal-demo">Show modal</button>
 *  <div data-rds-lightbox id="modal-demo">
 *    <button type="button" data-rds-lightbox-close>
 *      ...
 *    </button>
 *    ...
 *  </div>
 */

const Lightbox = {
  KEYCODE: 27, // esc key
  KEYCODE_TAB: 9, // tab key
  SELECTOR: 'rds-lightbox-target',
  MODAL: 'rds-lightbox',
  CLASS_VISIBLE: 'visible',
  CLASS_BODY: 'rds-body--with-lightbox-open',
  CLOSE_SELECTOR: 'rds-lightbox-close',
  CLASS_CONTAINER: 'rds-lightbox__container',
  BODY: 'body',

  init: function() {
    $$(`[data-${this.SELECTOR}]`).forEach(item => {
      item.addEventListener('click', e => this.handleClick.call(this, e, item))
    })

    $$(`[data-${this.CLOSE_SELECTOR}]`).forEach(item => {
      item.addEventListener('click', e => this.handleClickClose.call(this, e))
    })
  },

  handleClick: function(e, item) {
    e.preventDefault()
    this.showModal(item)
  },

  handleClickClose: function(e) {
    e.preventDefault()
    this.hideModal()
  },

  handleKeydown: function(e, item) {
    if (e.keyCode === this.KEYCODE) {
      this.hideModal()
      e.stopPropagation()
      e.preventDefault()
    }

    if (e.keyCode === this.KEYCODE_TAB) {
      this.trapFocus(e, item)
    }
  },

  trapFocus: function(e, item) {
    const focusableElements = 'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
    const firstFocusableElement = item.querySelectorAll(focusableElements)[0]
    const focusableContent = item.querySelectorAll(focusableElements)
    const lastFocusableElement = focusableContent[focusableContent.length - 1]

    // if shift key pressed for shift + tab combination
    if (e.shiftKey) {
      if (document.activeElement === firstFocusableElement) {
        lastFocusableElement.focus()
        e.preventDefault()
      }
    } else {
      // if tab key is pressed
      if (document.activeElement === lastFocusableElement) {
        firstFocusableElement.focus()
        e.preventDefault()
      }
    }
  },

  clickAway: function(e) {
    if ($(`.${this.CLASS_VISIBLE}`)) {
      let container = $(`.${this.CLASS_VISIBLE}`).querySelector(`.${this.CLASS_CONTAINER}`)

      if (container && !container.contains(e.target)) {
        this.hideModal()
        $(`${this.BODY}`).removeEventListener('click', () => this.clickAway)
      }
    }
  },

  accessibility: function(modal) {
    const focusableElements = 'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
    const firstFocusableElement = modal.querySelectorAll(focusableElements)[0]
    firstFocusableElement.focus()
    modal.addEventListener('keydown', e => this.handleKeydown.call(this, e, modal))
  },

  showModal: function(item) {
    const modal = $(`${item.dataset.rdsLightboxTarget}`)

    modal.classList.add(`${this.CLASS_VISIBLE}`)
    $(`${this.BODY}`).classList.add(`${this.CLASS_BODY}`)

    // click outside modal to close it
    $(`.${this.CLASS_VISIBLE}`).addEventListener('click', e => this.clickAway.call(this, e))

    // accessibility requirements
    this.accessibility(modal)
  },

  hideModal: function() {
    $$(`[data-${this.MODAL}]`).forEach(item => {
      item.classList.remove(`${this.CLASS_VISIBLE}`)
    })
    $(`${this.BODY}`).classList.remove(`${this.CLASS_BODY}`)
  }
}

export default Lightbox
