const $$ = document.querySelectorAll.bind(document)

/**
 * Tooltip with keyboard support
 * Use:
 *  <button data-rds-tooltip aria-expanded="<true|false>">
 *    <span data-rds-tooltip-close></span>
 *  </button>
 *  <span data-rds-tooltip-trigger></span>
 */

const Tooltip = {
  SELECTOR: 'rds-tooltip-trigger',
  CLOSE_SELECTOR: 'rds-tooltip-close',
  COMPONENT: 'rds-tooltip',
  KEYCODE_ESC: 27,
  KEYCODE_TAB: 9,

  init: function(selector = this.SELECTOR) {
    this.SELECTOR = selector

    $$(`[data-${this.SELECTOR}]`).forEach(item => {
      item.addEventListener('click', e => this.handleClick.call(this, e, item))
    })

    $$(`[data-${this.CLOSE_SELECTOR}]`).forEach(item => {
      item.addEventListener('click', e => this.handleClickClose.call(this, e, item))
    })
  },

  handleClick: function(e, item) {
    e.preventDefault()
    this.showTooltip(item)
  },

  handleClickClose: function(e, item) {
    e.preventDefault()
    this.hideTooltip(item)
  },

  handleKeydown: function(e, item) {
    if (e.keyCode === this.KEYCODE_ESC) {
      this.hideTooltip(item)
      e.stopPropagation()
      e.preventDefault()
    }
    if (e.keyCode === this.KEYCODE_TAB) {
      this.trapFocus(e, item)
    }
  },

  trapFocus: function(e, item) {
    const focusableElements = 'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
    const firstFocusableElement = item.querySelectorAll(focusableElements)[0]
    const focusableContent = item.querySelectorAll(focusableElements)
    const lastFocusableElement = focusableContent[focusableContent.length - 1]
    // if shift key pressed for shift + tab combination
    if (e.shiftKey) {
      if (document.activeElement === firstFocusableElement) {
        lastFocusableElement.focus()
        e.preventDefault()
      }
    } else {
      // if tab key is pressed
      if (document.activeElement === lastFocusableElement) {
        firstFocusableElement.focus()
        e.preventDefault()
      }
    }
  },

  accessibility: function(tooltip) {
    const focusableElements = 'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
    const firstFocusableElement = tooltip.querySelectorAll(focusableElements)[0]
    firstFocusableElement.focus()
    tooltip.addEventListener('keydown', e => this.handleKeydown.call(this, e, tooltip))
  },

  showTooltip: function(item) {
    const tooltip = item.closest(`[data-${this.COMPONENT}]`)
    tooltip.classList.add('active')
    tooltip.setAttribute('aria-expanded', 'true')
    // accessibility requirements
    this.accessibility(tooltip)
  },

  hideTooltip: function(item) {
    const tooltip = item.closest(`[data-${this.COMPONENT}]`)
    tooltip.classList.remove('active')
    tooltip.setAttribute('aria-expanded', 'false')
  }
}

export default Tooltip
