const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const IgnoreEmitPlugin = require('ignore-emit-webpack-plugin')
const path = require('path')
const { currentVersion, entries } = require('./webpack_utils')

module.exports = {
  mode: 'production',
  entry: entries,
  output: {
    path: path.resolve(__dirname, '../dist/' + currentVersion)
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: '[name].css',
      chunkFilename: '[id].css'
    }),
    new IgnoreEmitPlugin([ 'rds-icons.js', 'rds-reset.js', 'rds-variables.css', 'rds-variables.js', 'rds-foundations.js', 'rds-components.js' ])
  ],
  module: {
    rules: [
      {
        test: /\.(sc|c)ss$/,
        loader: [
          MiniCssExtractPlugin.loader,
          'css-loader',
          {
            loader: 'postcss-loader',
            options: {
              autoprefixer: {
                browsers: ['> 1%', 'last 2 versions', 'ie >= 11']
              },
              sourceMap: true,
              ident: 'postcss',
              plugins: () => [require('autoprefixer')]
            }
          },
          {
            loader: 'sass-loader',
            options: {
              sassOptions: {
                includePaths: ['src/styles']
              }
            }
          }
        ],
        sideEffects: true
      }
    ]
  }
}
